#ifndef INCL_RS232C_H
#define INCL_RS232C_H

#include "device.h"
#include "misc.h"
#include "if/ifcommon.h"

#define	BAUDRATE_DEFAULT	4		// 1200bps
#define	COM_DEFAULT			0		// disabled
#define	DATABUF_SIZE		65536

typedef struct {
	int baudrate;
	int comport;
} RS232C_CONFIG;

// ---------------------------------------------------------------------------

class Rs232c : public Device
{
public:
	enum
	{
		reset= 0, out20, out21, out30, out6f,
		in20 = 0, in21, in6f
	};

public:
	Rs232c();
	~Rs232c();
	
	bool Init(IIOAccess* access, IScheduler* _schduler, uint prxrdy, uint prequest);

	bool Open(RS232C_CONFIG* pconfig);
	void Close();

	// IDevice Method
	const Descriptor* IFCALL GetDesc() const { return &descriptor; }
	char* GetMessage(void) { return message; }
	
	// I/O port functions
	void IOCALL Reset(uint=0, uint=0);
	void IOCALL Out20(uint, uint);
	void IOCALL Out21(uint, uint);
	void IOCALL Out30(uint, uint);
	void IOCALL Out6f(uint, uint);
	uint IOCALL In20(uint);
	uint IOCALL In21(uint);
	uint IOCALL In6f(uint);

private:
	bool SetMode(int baudrate);
	void SetData(byte* data, uint count);
	uint ThreadMain();
	static uint __stdcall ThreadEntry(LPVOID arg);
	void IOCALL TimeEvent(uint);
	bool IsRs232cEnabled(void) 
	{ 
		return ((port30 & 0x20) != 0 && hcom != INVALID_HANDLE_VALUE);
	}

	enum Mode { clear=0, async, sync1, sync2, sync };
	enum Parity { none='N', odd='O', even='E' };
	enum
	{
		TXRDY	= 0x01,
		RXRDY	= 0x02,
		TXE		= 0x04,
		PE		= 0x08,
		OE		= 0x10,
		FE		= 0x20,
		SYNDET	= 0x40,
		DSR		= 0x80,
	
		ssrev	= 1,
	};

	uint clock;
	Parity parity;
	uint datalen;
	uint stop;
	Mode mode;
	uint status;
	bool rxen;
	bool txen;

	IIOAccess* access;
	IScheduler* scheduler;
	uint prxrdy;
	uint prequest;

	char message[256];

	RS232C_CONFIG config;
	uint port30;

	CRITICAL_SECTION cs;
	HANDLE	hcom;
	DCB		dcb;
	HANDLE	hthread;
	uint	idthread;
	bool	running;
	bool	checking;
	bool	setting;
	byte	databuf[DATABUF_SIZE];
	int		statuscount;
	int		datacount;
	int		datapos;
	COMSTAT comstat;
	DWORD	tick;

	static const Descriptor descriptor;
	static const OutFuncPtr outdef[];
	static const InFuncPtr  indef[];
};

#endif // INCL_RS232C_H
