#include "headers.h"
#include "config.h"
#include "resource.h"

// ---------------------------------------------------------------------------

ConfigRs232c::ConfigRs232c()
{
	gate.SetDestination(PageGate, this);
}

bool ConfigRs232c::Init(HINSTANCE _hinst, Rs232c* _rs232c)
{
	hinst = _hinst;
	rs232c = _rs232c;

	char	szBuf[_MAX_PATH];

	GetModuleFileName(0, szBuf, _MAX_PATH);
	LPTSTR filepart;
	GetFullPathName(szBuf, _MAX_PATH, szIniFile, &filepart);
	*filepart = '\0';
	lstrcpy(filepart, "M88.INI");

	config.baudrate = GetPrivateProfileInt("RS232C", "BAUDRATE", BAUDRATE_DEFAULT, szIniFile);
	config.comport  = GetPrivateProfileInt("RS232C", "COM_PORT", COM_DEFAULT, szIniFile);
	config.baudrate = Limit(config.baudrate, 8, 0);
	config.comport  = Max(config.comport, 0);

	return true;
}

bool IFCALL ConfigRs232c::Setup(IConfigPropBase* _base, PROPSHEETPAGE* psp)
{
	base = _base;
	
	memset(psp, 0, sizeof(PROPSHEETPAGE));
	psp->dwSize = sizeof(PROPSHEETPAGE);
	psp->dwFlags = 0;
	psp->hInstance = hinst;
	psp->pszTemplate = MAKEINTRESOURCE(IDD_CONFIG);
	psp->pszIcon = 0;
	psp->pfnDlgProc = (DLGPROC) (void*) gate;
	psp->lParam = 0;
	return true;
}

BOOL ConfigRs232c::PageProc(HWND hdlg, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg)
	{
	case WM_INITDIALOG:
		InitDialog(hdlg);
		return TRUE;

	case WM_NOTIFY:
		switch (((NMHDR*) lp)->code)
		{
		case PSN_SETACTIVE:
			Update(hdlg);
			base->PageSelected(this);
			break;

		case PSN_APPLY:
			Apply(hdlg);
			base->Apply();
			return PSNRET_NOERROR;
		}
		return TRUE;
	case WM_COMMAND:
		return Command(hdlg, HWND(lp), HIWORD(wp), LOWORD(wp));
	}
	return FALSE;
}

BOOL CALLBACK ConfigRs232c::PageGate
(ConfigRs232c* config, HWND hwnd, UINT m, WPARAM w, LPARAM l)
{
	return config->PageProc(hwnd, m, w, l);
}

void ConfigRs232c::InitDialog(HWND hdlg)
{
	config_disp = config;
	SendDlgItemMessage(hdlg, IDC_PORT, EM_SETLIMITTEXT, 2, 0);
	SendDlgItemMessage(hdlg, IDC_PORT, TBM_SETRANGE, TRUE, MAKELONG(0, 99));
	SendDlgItemMessage(hdlg, IDC_PORTSPIN, UDM_SETRANGE, 0, MAKELONG(99, 0));
	char buf[8];
	for (int i=0; i<9; i++) 
	{
		sprintf(buf, "%d", 75 << i);
		SendDlgItemMessage(hdlg, IDC_BAUDRATE, CB_ADDSTRING, 0, (LPARAM)buf);
	}
}

void ConfigRs232c::Update(HWND hdlg)
{
	SetDlgItemInt(hdlg, IDC_PORT, config_disp.comport, false);
	SendDlgItemMessage(hdlg, IDC_BAUDRATE, CB_SETCURSEL, config_disp.baudrate, 0);
	SetDlgItemText(hdlg, IDC_STATUS, rs232c->GetMessage());
}

void ConfigRs232c::Apply(HWND hdlg)
{
	char	buf[16];

	config = config_disp;
	sprintf(buf, "%d", config.baudrate);
	WritePrivateProfileString("RS232C", "BAUDRATE", buf, szIniFile);
	sprintf(buf, "%d", config.comport);
	WritePrivateProfileString("RS232C", "COM_PORT", buf, szIniFile);

	if (rs232c) 
		rs232c->Open(&config);
	SetDlgItemText(hdlg, IDC_STATUS, rs232c->GetMessage());
}

BOOL ConfigRs232c::Command(HWND hdlg, HWND hwctl, UINT nc, UINT id)
{
	switch (id)
	{
	case IDC_BAUDRATE:
		if (nc == CBN_SELCHANGE)
		{
			int baud = SendDlgItemMessage(hdlg, IDC_BAUDRATE, CB_GETCURSEL, 0, 0);
			if (baud != config_disp.baudrate)
				base->PageChanged(hdlg);
			config_disp.baudrate = baud;
			return true;
		}
		break;
	case IDC_PORT:
		if (nc == EN_CHANGE)
		{
			int port = Limit(GetDlgItemInt(hdlg, IDC_PORT, 0, false), 100, 0);
			if (port != config_disp.comport)
				base->PageChanged(hdlg);
			config_disp.comport = port;
			return TRUE;
		}
		break;
	}
	return FALSE;
}
