#include "headers.h"
#include "if/ifcommon.h"
#include "if/ifguid.h"
#include "Rs232c.h"
#include "config.h"

#define EXTDEVAPI	__declspec(dllexport)

HINSTANCE hinst;

// ---------------------------------------------------------------------------

enum SpecialPort
{
	pint0 = 0x100, 
	pint1, pint2, pint3, pint4, pint5, pint6, pint7,
	pres,			// reset
	pirq,			// IRQ
	piack,			// interrupt acknowledgement
	vrtc,			// vertical retrace
	popnio,			// OPN ̓o̓|[g 1
	popnio2,		// OPN ̓o̓|[g 2 (A)
	psioin,			// SIO ֌W
	psioreq,
	ptimesync,
	portend
};

// ---------------------------------------------------------------------------

class Rs232cModule : public IModule
{
public:
	Rs232cModule();
	~Rs232cModule() {}

	bool Init(ISystem* system);
	void IFCALL Release();
	void* IFCALL QueryIF(REFIID) { return 0; }

private:
	Rs232c rs232c;
	ConfigRs232c cfg;
	
	ISystem* sys;
	IIOBus* bus;
	IConfigPropBase* pb;
};

Rs232cModule::Rs232cModule()
: bus(0), pb(0)
{
	cfg.Init(hinst, &rs232c);
}

bool Rs232cModule::Init(ISystem* _sys)
{
	sys = _sys;
	
	bus = (IIOBus*) sys->QueryIF(M88IID_IOBus1);
	pb = (IConfigPropBase*) sys->QueryIF(M88IID_ConfigPropBase);
	IIOAccess* access = (IIOAccess*) sys->QueryIF(M88IID_IOAccess1);
	IScheduler* sch = (IScheduler*) sys->QueryIF(M88IID_Scheduler);
	
	if (!bus || !pb || !access)
		return false;
	
	const static IIOBus::Connector c_rs232c[] =
	{
		{ pres, IIOBus::portout, Rs232c::reset },
		{ 0x20, IIOBus::portout, Rs232c::out20 },
		{ 0x21, IIOBus::portout, Rs232c::out21 },
		{ 0x30, IIOBus::portout, Rs232c::out30 },
		{ 0x6f, IIOBus::portout, Rs232c::out6f },
		{ 0x20, IIOBus::portin,  Rs232c::in20  },
		{ 0x21, IIOBus::portin,  Rs232c::in21  },
		{ 0x6f, IIOBus::portin,  Rs232c::in6f  },
		{ 0, 0, 0 }
	};
	if (!rs232c.Init(access, sch, pint0, psioin) || !bus->Connect(&rs232c, c_rs232c))
		return false;

	pb->Add(&cfg);

	rs232c.Open(cfg.GetConfig());

	return true;
}

void Rs232cModule::Release()
{
	rs232c.Close();

	if (bus)
		bus->Disconnect(&rs232c);
	if (pb)
		pb->Remove(&cfg);

	delete this;
}

// ---------------------------------------------------------------------------

//	Module 쐬
extern "C" EXTDEVAPI IModule* __cdecl M88CreateModule(ISystem* system)
{
	Rs232cModule* module = new Rs232cModule;
	
	if (module)
	{
		if (module->Init(system))
			return module;
		delete module;
	}
	return 0;
}

BOOL APIENTRY DllMain(HANDLE hmod, DWORD rfc, LPVOID)
{
	switch (rfc)
	{
	case DLL_PROCESS_ATTACH:
		hinst = (HINSTANCE) hmod;
		break;
	
	case DLL_THREAD_ATTACH:
		break;
	
	case DLL_THREAD_DETACH:
		break;
	
	case DLL_PROCESS_DETACH:
		break;
	}
	return true;
}
